using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using Unity.Entities;
using Unity.Jobs;
using Unity.Burst;
using Unity.Mathematics;
using UnityEngine.Jobs;
using Unity.Transforms;
using Unity.Collections;
using Unity.Physics;

public class AsteroidConstaintJobSystem : JobComponentSystem
{
	public struct Job : IJobForEach< AsteroidConstrained , PhysicsVelocity , Translation >
	{
		public float3 playerPosition;
		public float3 playerVelocity;
		public float distance;

		public void Execute( [ReadOnly]ref AsteroidConstrained asteroid , [ReadOnly]ref PhysicsVelocity physicsVelocity , ref Translation translation )
		{
			var pos = translation.Value;
			pos.y = 0.0f;
			translation.Value = pos;

			var vel = physicsVelocity.Linear;
			vel.y = 0.0f;
			physicsVelocity.Linear = vel;

			var dirToPlayer = playerPosition - translation.Value;
			var length = math.lengthsq(dirToPlayer);
			var maxDistance = distance;
			if (length > maxDistance* maxDistance && asteroid.positionConstrained)
			{
				
				translation.Value += maxDistance * 1.9f *math.normalizesafe(dirToPlayer);
				vel = math.normalizesafe(vel);
			}
		}
	}

	EntityQuery playerQuery;
	protected override void OnCreate()
	{
		base.OnCreate();
		playerQuery = EntityManager.CreateEntityQuery(ComponentType.ReadWrite<Translation>(), ComponentType.ReadOnly<Player>(), ComponentType.ReadOnly<PlayerMovementData>(), ComponentType.ReadOnly<PlayerScoring>());

	}
	protected override JobHandle OnUpdate(JobHandle inputDeps)
	{
		using (var playerTrans = playerQuery.ToComponentDataArray<Translation>(Allocator.TempJob))
		using (var playerVel = playerQuery.ToComponentDataArray<PlayerMovementData>(Allocator.TempJob))
		using (var playerScoring = playerQuery.ToComponentDataArray<PlayerScoring>(Allocator.TempJob))
		{
			//if (playerScoring[0].score < 200)
			if (playerScoring.Length>0)
			{
				Job job = new Job()
				{

					distance = (playerScoring[0].score < 200 ? LevellingData.Inst.asteroidDistance.Evaluate(playerScoring[0].score) : 100000000.0f),
					playerPosition = playerTrans[0].Value,
					playerVelocity = playerVel[0].currentVelocity
				};
				return job.Schedule(this, inputDeps);
			}
			return inputDeps;
		}
		
	}
}
