using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using Unity.Entities;
using Unity.Jobs;
using Unity.Burst;
using Unity.Mathematics;
using UnityEngine.Jobs;
using Unity.Transforms;
using Unity.Physics;

public class BlackHoleJob : JobComponentSystem
{

	public struct Job : IJobForEach< Gatherable , Translation >
	{
		public float3 blackHolePosition;
		public float deltaTime;
		public float speed;
		public void Execute(ref Gatherable gatherable,  ref Translation translation )
		{
			var direction = blackHolePosition - translation.Value;
			var directionMag = math.lengthsq(direction);
			var lateral = math.cross(direction, Vector3.up);
			translation.Value += lateral * deltaTime * speed*100.0f/directionMag;
			translation.Value += math.normalizesafe(direction) * deltaTime * speed;
			
		}
	}

	EntityQuery blackHoleQuery;
	BeginInitializationEntityCommandBufferSystem cmdBuffSystem;
	protected override void OnCreate()
	{
		base.OnCreate();
		blackHoleQuery = EntityManager.CreateEntityQuery(ComponentType.ReadOnly<Blackhole>(), ComponentType.ReadOnly<Translation>());
		cmdBuffSystem = World.GetOrCreateSystem<BeginInitializationEntityCommandBufferSystem>();
	}
	protected override JobHandle OnUpdate(JobHandle inputDeps)
	{
		using (var translation = blackHoleQuery.ToComponentDataArray<Translation>(Unity.Collections.Allocator.TempJob))
		using (var bHoles = blackHoleQuery.ToComponentDataArray<Blackhole>(Unity.Collections.Allocator.TempJob))
		{
			if (translation.Length > 0)
			{
				Job job = new Job()
				{
					blackHolePosition = translation[0].Value,
					speed = math.pow(bHoles[0].elapsedTime,bHoles[0].speed),
					deltaTime = Time.DeltaTime
				};
				return job.Schedule(this, inputDeps);
			}
		}
		return inputDeps;
	}
}
public class BlackHoleScalingJob : JobComponentSystem
{

	public struct Job : IJobForEach<Gatherable,Translation,CompositeScale>
	{
		public float3 blackHolePosition;
		public float deltaTime;
		public float speed;
		public void Execute(ref Gatherable gatherable, ref Translation translation, ref CompositeScale scale)
		{
			var direction = blackHolePosition - translation.Value;
			var directionMag = math.lengthsq(direction);
			if (directionMag < 120.0f)
			{
				scale.Value = float4x4.Scale(0.0f);
			}
		}
	}

	EntityQuery blackHoleQuery;
	BeginInitializationEntityCommandBufferSystem cmdBuffSystem;
	protected override void OnCreate()
	{
		base.OnCreate();
		blackHoleQuery = EntityManager.CreateEntityQuery(ComponentType.ReadOnly<Blackhole>(), ComponentType.ReadOnly<Translation>());
		cmdBuffSystem = World.GetOrCreateSystem<BeginInitializationEntityCommandBufferSystem>();
	}
	protected override JobHandle OnUpdate(JobHandle inputDeps)
	{
		using (var translation = blackHoleQuery.ToComponentDataArray<Translation>(Unity.Collections.Allocator.TempJob))
		using (var bHoles = blackHoleQuery.ToComponentDataArray<Blackhole>(Unity.Collections.Allocator.TempJob))
		{
			if (translation.Length > 0)
			{
				Job job = new Job()
				{
					blackHolePosition = translation[0].Value,
					speed = math.pow(bHoles[0].elapsedTime, bHoles[0].speed),
					deltaTime = Time.DeltaTime
				};
				return job.Schedule(this, inputDeps);
			}
		}
		return inputDeps;
	}
}
public class BlackHoleInitSystem : ComponentSystem
{
	protected override void OnUpdate()
	{
		Entities.WithAll<Blackhole>().ForEach((Entity bhEnt, ref Blackhole bHole)=>
		{
			bHole.elapsedTime += Time.DeltaTime;
		Entities.ForEach((Entity ent, ref PhysicsVelocity physicsVelocity, ref Gatherable gatherable) =>
		{
			PostUpdateCommands.RemoveComponent<PhysicsVelocity>(ent);
		});
		});
	}
}