using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using Unity.Entities;
using Unity.Mathematics;
using Unity.Transforms;

public static  class LineHelpers
{
	// Find the point of intersection between
	// the lines p1 --> p2 and p3 --> p4.
	public static bool FindIntersection(
	float3 p1, float3 p2, float3 p3, float3 p4,
	out float3 intersection)
	{
		bool segments_intersect;
		// Get the segments' parameters.
		float dx12 = p2.x - p1.x;
		float dy12 = p2.z - p1.z;
		float dx34 = p4.x - p3.x;
		float dy34 = p4.z - p3.z;

		// Solve for t1 and t2
		float denominator = (dy12 * dx34 - dx12 * dy34);

		float t1 =
			((p1.x - p3.x) * dy34 + (p3.z - p1.z) * dx34)
				/ denominator;
		if (float.IsInfinity(t1))
		{
			// The lines are parallel (or close enough to it).
			segments_intersect = false;
			intersection = new float3(float.NaN, float.NaN, float.NaN);
			return segments_intersect; 
		}
		

		float t2 =
			((p3.x - p1.x) * dy12 + (p1.z - p3.z) * dx12)
				/ -denominator;

		// Find the point of intersection.
		intersection = new float3(p1.x + dx12 * t1, 0.0f, p1.z + dy12 * t1);

		// The segments intersect if t1 and t2 are between 0 and 1.
		segments_intersect =
			((t1 > 0) && (t1 < 1) &&
			 (t2 > 0) && (t2 < 1));

		return segments_intersect;
	}
}
