using Unity.Mathematics;
using Unity.Transforms;
using Unity.Collections;
using System.Collections;
using System.Collections.Generic;
using Unity.Entities;
using UnityEngine;
using UnityEngine.InputSystem;

[System.Serializable]
public struct InputMovementComponent : IComponentData
{
	public float2 leftStick;
	public float2 rightStick;
}


[System.Serializable]
public struct InputAbilitiesComponent : IComponentData
{
	public enum InputPressType
	{
		eNone,

		eDown,
		ePressed,
		eUp
	}

	public InputPressType Jump;
	public InputPressType Crouch;
	public InputPressType Reset;
    public InputPressType GrabLeft;
    public InputPressType GrabRight;
    public InputPressType Charge;
    public InputPressType ChangeCamera;
    public static void UpdatePressType(ref InputPressType input)
	{
		switch (input)
		{
			case InputPressType.eDown:
				input = InputPressType.ePressed;
				break;
			case InputPressType.eUp:
				input = InputPressType.eNone;
				break;
		}
	}
	public static void SetPressType(ref InputPressType input, bool isDown)
	{
		if (isDown)
		{
			switch (input)
			{
				case InputPressType.eNone:
					input = InputPressType.eDown;
					break;
				case InputPressType.eDown:
					input = InputPressType.ePressed;
					break;
				case InputPressType.eUp:
					input = InputPressType.eDown;
					break;
			}
		}
		else
		{
			switch (input)
			{
				case InputPressType.eDown:
					input = InputPressType.eUp;
					break;
				case InputPressType.ePressed:
					input = InputPressType.eUp;
					break;
				case InputPressType.eUp:
					input = InputPressType.eNone;
					break;
			}
		}
	}
}


//this is on the entity created via the game object with UnityEngine.InputSystem.PlayerInput on it.
[System.Serializable]
public struct MovementRelayComponent : IComponentData
{

}
[RequiresEntityConversion]
public class InputComponentConverted: MonoBehaviour, IConvertGameObjectToEntity
{
	public InputMovementComponent inputMovement;
	public InputAbilitiesComponent abilitiesComponent;
	PlayerInput playerInput;
	InputAction moveAction;
	InputAction lookAction;

	Entity thisEntity;

	void Awake()
	{
		playerInput = GetComponent<PlayerInput>();
		moveAction = playerInput.actions.FindAction("Move");
		lookAction = playerInput.actions.FindAction("Look");

	}

	void Update()
	{
		inputMovement.leftStick = moveAction.ReadValue<Vector2>();
		inputMovement.rightStick = lookAction.ReadValue<Vector2>();
		
        if (thisEntity != Entity.Null)
        {
            World.Active.EntityManager.SetComponentData(thisEntity, inputMovement);
            World.Active.EntityManager.SetComponentData(thisEntity, abilitiesComponent);
            InputAbilitiesComponent.UpdatePressType(ref abilitiesComponent.Jump);
        }
	}			

	public void Convert(Entity entity, EntityManager dstManager, GameObjectConversionSystem conversionSystem)
    {
        playerInput = GetComponent<PlayerInput>();
        thisEntity = entity;
        dstManager.AddComponentData(entity, inputMovement);
		dstManager.AddComponentData(entity, abilitiesComponent);
		dstManager.AddComponentData(entity, new MovementRelayComponent());

    }
	public void OnJump(InputValue value)
	{
		bool isDown = value.isPressed;
		InputAbilitiesComponent.SetPressType(ref abilitiesComponent.Jump, isDown);
	}
	public void OnCrouch(InputValue value)
	{
		bool isDown = value.isPressed;
		InputAbilitiesComponent.SetPressType(ref abilitiesComponent.Crouch, isDown);
	}
    public void OnGrabLeft(InputValue value)
    {
        bool isDown = value.isPressed;
        InputAbilitiesComponent.SetPressType(ref abilitiesComponent.GrabLeft, isDown);
    }
    public void OnGrabRight(InputValue value)
    {
        bool isDown = value.isPressed;
        InputAbilitiesComponent.SetPressType(ref abilitiesComponent.GrabRight, isDown);
    }
    public void OnReset(InputValue value)
	{
		bool isDown = value.isPressed;
		InputAbilitiesComponent.SetPressType(ref abilitiesComponent.Reset, isDown);
	}
    public void OnCharge(InputValue value)
    {
        bool isDown = value.isPressed;
        InputAbilitiesComponent.SetPressType(ref abilitiesComponent.Charge, isDown);
    }
    public void OnChangeCam(InputValue value)
    {
        bool isDown = value.isPressed;
        InputAbilitiesComponent.SetPressType(ref abilitiesComponent.ChangeCamera, isDown);
    }
}


public class InputFilterSystem : ComponentSystem
{
	EntityQuery inputQuery;
	protected override void OnCreate()
	{
		base.OnCreate();
		inputQuery = EntityManager.CreateEntityQuery(typeof(InputMovementComponent), ComponentType.Exclude<MovementRelayComponent>(), typeof (InputAbilitiesComponent));

	}
	protected override void OnUpdate()
	{
		Entities.WithAllReadOnly<MovementRelayComponent>().ForEach((ref InputMovementComponent relayComponent, ref InputAbilitiesComponent abilities) =>
		{
			var movementComponent = relayComponent;
			var inputAbilitiesComponent = abilities;
			Entities.With(inputQuery).ForEach((Entity ent) =>
			{				
				PostUpdateCommands.SetComponent(ent, movementComponent);
				PostUpdateCommands.SetComponent(ent, inputAbilitiesComponent);
			});
		});
	}
}
