using Unity.Mathematics;
using Unity.Transforms;
using System.Collections;
using System.Collections.Generic;
using Unity.Entities;
using UnityEngine;

[System.Serializable]
public struct MoonSpawner : IComponentData
{
	public bool shouldRotate;
	public float offsetMin;
	public float offsetMax;
}

[System.Serializable]
public struct MoonPrefabs : IBufferElementData
{
	public Entity prefabEnt;
}
[RequiresEntityConversion]
public class MoonSpawnerAuthoring: MonoBehaviour, IConvertGameObjectToEntity, IDeclareReferencedPrefabs
{
	public List<GameObject> moonPrefabs;
    public MoonSpawner moonSpawner;
    public void Convert(Entity entity, EntityManager dstManager, GameObjectConversionSystem conversionSystem)
    {

		var buffer = dstManager.AddBuffer<MoonPrefabs>(entity);
		foreach(var prefab in moonPrefabs)
		{
			Entity moonEntity;
			moonEntity = conversionSystem.GetPrimaryEntity(prefab);
			Debug.Assert(moonEntity != Entity.Null, "Failed to get moon entity");
			buffer.Add(new MoonPrefabs() { prefabEnt = moonEntity });
		}
		
		dstManager.AddComponentData(entity, moonSpawner);
		
	}

	public void DeclareReferencedPrefabs(List<GameObject> referencedPrefabs)
	{
		foreach (var prefab in moonPrefabs)
		{
			referencedPrefabs.Add(prefab);
		}
	}
	public void OnDrawGizmosSelected()
	{
#if UNITY_EDITOR
		UnityEditor.Handles.DrawWireDisc(transform.position, Vector3.up, moonSpawner.offsetMin);
		UnityEditor.Handles.DrawWireDisc(transform.position, Vector3.up, moonSpawner.offsetMax);
#endif
	}
}

public class MoonSpawningSystem : ComponentSystem
{
	protected override void OnUpdate()
	{
		var buffs = GetBufferFromEntity<MoonPrefabs>(true);
		Entities.ForEach((Entity thisEnt, ref MoonSpawner moonSpawner, ref Translation trans) =>
		{
			int num = UnityEngine.Random.Range(0, 10);
			var prefabArray = buffs[thisEnt];
			for (int i = 0; i < 5; i++)
			{
				var moonEntityToInstantiate = prefabArray[UnityEngine.Random.Range(0, prefabArray.Length)].prefabEnt;
				var moonEntity = PostUpdateCommands.Instantiate(moonEntityToInstantiate);
				
				var direction = (float3)UnityEngine.Random.onUnitSphere;
				direction.y = 0.0f;
				direction = math.normalize(direction);
				
				var newOrbiting = new Orbiting();
				newOrbiting.parentPos = trans.Value;
				
				float scale = UnityEngine.Random.Range(20, 200.0f);
				
				PostUpdateCommands.AddComponent<Scale>(moonEntity, new Scale() { Value = scale });

				float distance = Mathf.Lerp(moonSpawner.offsetMin, moonSpawner.offsetMax, UnityEngine.Random.value);

				newOrbiting.speed = moonSpawner.shouldRotate ? distance*0.1f :0.0f;

				float3 position = trans.Value + direction * distance;
				Debug.Log("Spawning moon at position: " + position + "distance: " + distance);
				PostUpdateCommands.AddComponent<Translation>(moonEntity, new Translation() { Value = position });
				PostUpdateCommands.AddComponent<Orbiting>(moonEntity, newOrbiting);

			}
			PostUpdateCommands.RemoveComponent<MoonSpawner>(thisEnt);
		});
	}
}