using Unity.Mathematics;
using Unity.Transforms;
using System.Collections;
using System.Collections.Generic;
using Unity.Entities;
using UnityEngine;

[System.Serializable]
public struct PlayerCamera : IComponentData
{
	public float moveSpeed;
	public float boundary;
}
[RequiresEntityConversion]
public class PlayerCameraAuthoring: MonoBehaviour, IConvertGameObjectToEntity
{
    public PlayerCamera playerCamera;
	
    public void Convert(Entity entity, EntityManager dstManager, GameObjectConversionSystem conversionSystem)
    {
        dstManager.AddComponentData(entity, playerCamera);
    }
}

public class PlayerCameraFollowSystem : ComponentSystem
{
	protected override void OnUpdate()
	{
		Entities.ForEach((ref PlayerCamera playerCamera) =>
		{
			var cam = Camera.main;
			var playerCam = playerCamera;
			Entities.ForEach((ref Player player, ref Translation trans, ref PlayerScoring scoring) =>
			{
				var playerPos = trans.Value;

				var startScreenPos = cam.WorldToScreenPoint(trans.Value);
				var xPercentage = startScreenPos.x / cam.scaledPixelWidth;
				var yPercentage = startScreenPos.y / cam.scaledPixelHeight;
				var position = cam.transform.position;
				position.y = LevellingData.Inst.camHeightCurve.Evaluate(scoring.score);
				cam.transform.position = position;
				
				if (xPercentage>1.0f- playerCam.boundary)
				{
					var screenPos = cam.WorldToScreenPoint(trans.Value);
					var screenBoundary = cam.ScreenToWorldPoint(new Vector3(cam.scaledPixelWidth * (1.0f - playerCam.boundary), screenPos.y, screenPos.z));
					var diff = (Vector3)trans.Value - screenBoundary;					
					cam.transform.position += diff;
				}
				if (xPercentage < playerCam.boundary)
				{
					var screenPos = cam.WorldToScreenPoint(trans.Value);
					var screenBoundary = cam.ScreenToWorldPoint(new Vector3(cam.scaledPixelWidth * (playerCam.boundary), screenPos.y, screenPos.z));
					var diff = (Vector3)trans.Value - screenBoundary;					
					cam.transform.position += diff;
				}

				if (yPercentage > 1.0f - playerCam.boundary)
				{
					var screenPos = cam.WorldToScreenPoint(trans.Value);
					var screenBoundary = cam.ScreenToWorldPoint(new Vector3(screenPos.x, cam.scaledPixelHeight * (1.0f - playerCam.boundary), screenPos.z));
					var diff = (Vector3)trans.Value - screenBoundary;					
					cam.transform.position += diff;
				}
				if (yPercentage < playerCam.boundary)
				{
					var screenPos = cam.WorldToScreenPoint(trans.Value);
					var screenBoundary = cam.ScreenToWorldPoint(new Vector3(screenPos.x, cam.scaledPixelHeight * (playerCam.boundary), screenPos.z));
					var diff = (Vector3)trans.Value - screenBoundary;					
					cam.transform.position += diff;
				}
			});
		});
	}
}