using Unity.Transforms;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using Unity.Entities;
using Unity.Mathematics;

public class PlayerMovementSystem : ComponentSystem
{
	protected override void OnUpdate()
	{
		Entities.ForEach((ref Translation translation, ref InputMovementComponent inputMovementComponent, ref PlayerMovementData playerMovement, ref PlayerScoring scoring) =>
		{
			var currentSpeed = playerMovement.currentVelocity;
			currentSpeed.x += inputMovementComponent.leftStick.x * Time.DeltaTime * playerMovement.acceleration;
			currentSpeed.z += inputMovementComponent.leftStick.y * Time.DeltaTime * playerMovement.acceleration;
			var sqrLength = math.lengthsq(currentSpeed);

			if (sqrLength > 0.0f)
			{
				float actualSpeed = math.length(currentSpeed);
				float deceleration = math.min(actualSpeed, Time.DeltaTime * playerMovement.deceleration);
				currentSpeed -= math.normalize(currentSpeed) * deceleration;

			}

			if (sqrLength > playerMovement.maxSpeed * playerMovement.maxSpeed)
			{
				currentSpeed = math.normalize(currentSpeed) * playerMovement.maxSpeed;
			}

			playerMovement.currentVelocity = currentSpeed;

			var playerPosition = translation.Value;
			var score = scoring.score;

			Entities.ForEach((ref PlayerTether tether, ref Translation tetherPosition) =>
			{
				if (tether.topLevel > score)
				{
					var midPoint = tetherPosition.Value;
					var range = tether.range;
					var pullForce = tether.pullForce;
					var directionToMidPoint = tetherPosition.Value - playerPosition;
					if (math.lengthsq(directionToMidPoint) > range * range)
					{
						var distance = math.length(directionToMidPoint);
						var direction = math.normalize(directionToMidPoint);

						float outOfRange = distance - range;
						float outOfRangeMultiplier = outOfRange / (range * 0.1f);
						float speedDot = math.dot(-direction, math.normalizesafe(currentSpeed));
						if (speedDot > 0.0f)
						{
							//Debug.LogFormat("speed: {0} outOfRangeMultiplier: {1}", speedDot, outOfRangeMultiplier);
							currentSpeed *= (1.0f - math.clamp(speedDot * outOfRangeMultiplier, 0.0f, 1.0f));
						}
					}
				}

			});
			translation.Value += currentSpeed;
		});
	}
}