using Unity.Mathematics;
using Unity.Transforms;
using System.Collections;
using System.Collections.Generic;
using Unity.Entities;
using UnityEngine;

[System.Serializable]
public struct PlayerScoring : IComponentData
{
	public float score;
	public float desiredScore;
	public float scoreIncreaseSpeed;
}
[System.Serializable]
public struct IncrementPlayerScoring : IComponentData
{
	public int incrementAmount;
}
[RequiresEntityConversion]
public class PlayerScoringAuthoring: MonoBehaviour, IConvertGameObjectToEntity
{
    public PlayerScoring playerScoring;
    public void Convert(Entity entity, EntityManager dstManager, GameObjectConversionSystem conversionSystem)
    {
        dstManager.AddComponentData(entity, playerScoring);
    }
}

public class IncrementPlayerScoringSystem : ComponentSystem
{
	protected override void OnUpdate()
	{
		Entities.ForEach((ref PlayerScoring playerScoring) =>
		{
			if (playerScoring.score < playerScoring.desiredScore)
			{
				playerScoring.score += Time.DeltaTime * playerScoring.scoreIncreaseSpeed;
				playerScoring.score = math.min(playerScoring.score, playerScoring.desiredScore);
			}
			LevellingData.Inst.currentScore = playerScoring.score;
			int increment = 0;
			Entities.ForEach((Entity ent, ref IncrementPlayerScoring incrementPlayerScoring) =>
			{
				increment += incrementPlayerScoring.incrementAmount;
				PostUpdateCommands.DestroyEntity(ent);
			});
			playerScoring.desiredScore += increment;
			if (increment != 0)
			{
				Debug.Log("New score: " + playerScoring.desiredScore);
			}
		});
	}
}