using Unity.Transforms;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using Unity.Entities;
using Unity.Mathematics;

public class AsteroidSpawningSystem : ComponentSystem
{
	protected override void OnUpdate()
	{
		Entities.ForEach((Entity spawningEnt, ref AsteroidSpawnRequest asteroidSpawnRequest) =>
		{
			Entity entityToClone = Entity.Null;
			Debug.LogFormat("Going to with scale: {0}", asteroidSpawnRequest.scale);

			var spawnScale = asteroidSpawnRequest.scale;
			bool shouldScale = true;
			bool gatherable = true;
			Entities.WithAll<SpawnListTag>().ForEach((Entity spawnEnt) =>
			{
				var spawnListBuffer = GetBufferFromEntity<SpawnList>(true);
				var buff = spawnListBuffer[spawnEnt];
				for (int i = buff.Length-1; i >=0; i--)
				{
					var spawn = buff[i];
					if (spawn.sizeLimit < spawnScale)
					{
						entityToClone = spawn.ent;
						shouldScale = spawn.alterSize;
						gatherable = !spawn.notGatherable;
						break;
					}
				}
			});
			

			float3 position = asteroidSpawnRequest.position;
			float scale = asteroidSpawnRequest.scale;
			Entities.ForEach((ref AsteroidPrefab prefab) =>
			{
				var newAsteroid = PostUpdateCommands.Instantiate(entityToClone);
				PostUpdateCommands.SetComponent(newAsteroid, new Translation() { Value = position });
				if (shouldScale)
				{
					float4x4 scaleMat = float4x4.Scale(spawnScale);
					PostUpdateCommands.AddComponent(newAsteroid, new CompositeScale() { Value = scaleMat });
				}
				if (gatherable)
				{
					PostUpdateCommands.SetComponent(newAsteroid, new Gatherable() { baseMass = spawnScale });
				}
				else
				{
					PostUpdateCommands.AddComponent<ParticlePoolRequest>(newAsteroid, new ParticlePoolRequest() { particleId = (int)ParticlePoolAuthoring.Particles.blackHole, position = position });
				}
			});
			PostUpdateCommands.DestroyEntity(spawningEnt);
		});
	}
}