using Unity.Transforms;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using Unity.Entities;
using Unity.Jobs;
using Unity.Collections;
using Unity.Mathematics;


public class AttractionPointSolidifySystem : JobComponentSystem
{	
	EntityQuery gatherableQuery, attractionPointQuery;

	BeginInitializationEntityCommandBufferSystem cmdBuffSystem;
	protected override void OnCreate()
	{
		base.OnCreateManager();
		gatherableQuery = GetEntityQuery(  ComponentType.ReadWrite<GatherableAttractor>(), ComponentType.ReadWrite<Translation>());
		attractionPointQuery = GetEntityQuery(ComponentType.ReadWrite<AttractionPoint>(), ComponentType.ReadWrite<Translation>());
		cmdBuffSystem = World.GetOrCreateSystem<BeginInitializationEntityCommandBufferSystem>();
	}
	
	protected override JobHandle OnUpdate(JobHandle inputDeps)
	{
		JobHandle  AttractionPointHandle , GatherableAttractorHandle , GatherableTranslation, APTranslationHandle ;
		JobHandle gatherEntities, apEntitiesHandle; ;

		var AttractionPointArray = attractionPointQuery.ToComponentDataArray<AttractionPoint>(Allocator.TempJob, out AttractionPointHandle);
		var AttractionPointTranslationArray = attractionPointQuery.ToComponentDataArray<Translation>(Allocator.TempJob, out APTranslationHandle);
		var GatherableAttractorArray = gatherableQuery.ToComponentDataArray<GatherableAttractor>(Allocator.TempJob, out GatherableAttractorHandle);
		var gatherableTranslationArray = gatherableQuery.ToComponentDataArray<Translation>(Allocator.TempJob, out GatherableTranslation);
		
		var gatherableEntities = gatherableQuery.ToEntityArray(Allocator.TempJob, out gatherEntities);
		var apEntities = attractionPointQuery.ToEntityArray(Allocator.TempJob, out apEntitiesHandle);
		var dep = JobHandle.CombineDependencies(inputDeps,  AttractionPointHandle , GatherableAttractorHandle );
		dep = JobHandle.CombineDependencies(dep,gatherEntities, GatherableTranslation);
		dep = JobHandle.CombineDependencies(dep, APTranslationHandle, apEntitiesHandle);

		var job = new Job
		{
			gatherableEntities = gatherableEntities,
			 AttractionPointArray = AttractionPointArray
			, GatherableAttractorArray = GatherableAttractorArray
			, gatherableTranslationArray = gatherableTranslationArray
			, APTranslationArray = AttractionPointTranslationArray
			, cmdBuff = cmdBuffSystem.CreateCommandBuffer() 
			, elapsedTime = Time.ElapsedTime
			, APEntities = apEntities

		}.Schedule(dep);
		
		return job;
	}

	public struct Job : IJob
	{		
		[DeallocateOnJobCompletion]
		public NativeArray<Entity> gatherableEntities;

		[DeallocateOnJobCompletion]
		public NativeArray<Entity> APEntities;

		[DeallocateOnJobCompletion]
		public NativeArray<AttractionPoint> AttractionPointArray;

		[DeallocateOnJobCompletion]
		public NativeArray<GatherableAttractor> GatherableAttractorArray;

		[DeallocateOnJobCompletion]
		public NativeArray<Translation> APTranslationArray;

		[DeallocateOnJobCompletion]
		public NativeArray<Translation> gatherableTranslationArray;

		public EntityCommandBuffer cmdBuff;

		public double elapsedTime;
		public void Execute()
		{
			
			for (int i = 0; i < AttractionPointArray.Length; i++)
			{
				if (AttractionPointArray[i].creationTime + 2.0f < elapsedTime)
				{
					float mass = 0; ;
					var attractorPosition = APTranslationArray[i].Value;

					for (int j = 0; j < GatherableAttractorArray.Length; j++)
					{
						if (AttractionPointArray[i].id == GatherableAttractorArray[j].attractionPointId)
						{
							var gatherablePosition = gatherableTranslationArray[j].Value;
							var direction = attractorPosition - gatherablePosition;
							if (math.lengthsq(direction) < AttractionPointArray[i].requiredDistance * AttractionPointArray[i].requiredDistance)
							{
								mass += GatherableAttractorArray[j].mass;
							}
						}
					}

					//if we're over mass.
					

					if (mass > AttractionPointArray[i].requiredMass)
					{
						cmdBuff.DestroyEntity(APEntities[i]);
						var newent = cmdBuff.CreateEntity();
						cmdBuff.AddComponent<AsteroidSpawnRequest>(newent, new AsteroidSpawnRequest() { position = attractorPosition, scale =math.sqrt(mass) });
						cmdBuff.AddComponent<ParticlePoolRequest>(newent, new ParticlePoolRequest() { particleId = (int)ParticlePoolAuthoring.Particles.explode, position = attractorPosition });

						var scoringEnt = cmdBuff.CreateEntity();
						cmdBuff.AddComponent(scoringEnt, new IncrementPlayerScoring() { incrementAmount = Mathf.CeilToInt(AttractionPointArray[i].requiredMass) });

						for (int j = 0; j < GatherableAttractorArray.Length; j++)
						{
							if (AttractionPointArray[i].id == GatherableAttractorArray[j].attractionPointId)
							{
								var gatherablePosition = gatherableTranslationArray[j].Value;
								var direction = attractorPosition - gatherablePosition;
								if (math.lengthsq(direction) < AttractionPointArray[i].requiredDistance * AttractionPointArray[i].requiredDistance)
								{
									cmdBuff.DestroyEntity(gatherableEntities[j]);

								}
								else
								{
									cmdBuff.RemoveComponent<GatherableAttractor>(gatherableEntities[j]);
								}
							}
						}
					}
					else if(AttractionPointArray[i].creationTime + 4.0f < elapsedTime)
					{
						cmdBuff.DestroyEntity(APEntities[i]);
					}
				}
			}
		}
	}
}