using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using Unity.Entities;
using Unity.Jobs;
using Unity.Burst;
using Unity.Mathematics;
using UnityEngine.Jobs;
using Unity.Transforms;
using Unity.Collections;


public class TrailGatheringSystem : JobComponentSystem
{
	EntityQuery trailLoopQuery, gatherableQuery;
	int runningId;
	BeginInitializationEntityCommandBufferSystem bufferSystem;	
	
	public struct GatherableJob : IJobForEachWithEntity<Gatherable, Translation, CompositeScale>
	{
		[DeallocateOnJobCompletion, ReadOnly, NativeDisableParallelForRestriction]
		public NativeArray<Entity> trailEntities;

		[NativeDisableParallelForRestriction, ReadOnly]
		public BufferFromEntity<TrailLoopPoint> pointBuffers;

		[ReadOnly]
		public ComponentDataFromEntity<GatherableAttractor> attractors;
		
		public EntityCommandBuffer.Concurrent buffer;
		public int attractionPointId;
		public float3 midPoint;

		public void Execute(Entity ent, int index, [ReadOnly]ref Gatherable gatherable, [ReadOnly] ref Translation translation, [ReadOnly] ref CompositeScale scale)
		{
			for (int i = 0; i < trailEntities.Length; i++)
			{
				var points = pointBuffers[trailEntities[i]];

				var testPointStart = translation.Value;
				var testPointEnd = translation.Value + new float3(0.0f, 0.0f, 10000000.0f);
				int numIntercepts = 0;

				var cumulativePoints = points[0].position;

				for (int k = 1; k < points.Length; k++)
				{
					var point1 = points[k].position;
					var point2 = points[k - 1].position;
					float3 intersect;
					if (LineHelpers.FindIntersection(testPointStart, testPointEnd, point1, point2, out intersect))
					{
						numIntercepts++;
					}
				}

				if ((numIntercepts % 2) != 0)
				{
					//if (!attractors.Exists(ent))
					{
						buffer.AddComponent(index, ent, new GatherableAttractor() { attractPosition = midPoint, attractionPointId = attractionPointId, mass = gatherable.baseMass*scale.Value.c0.x* scale.Value.c0.x });
					}
				}
			}
		}
	}

	protected override void OnCreate()
	{
		base.OnCreate();
		trailLoopQuery = GetEntityQuery(ComponentType.ReadOnly<TrailLoop>(), ComponentType.ReadOnly<TrailFinished>());
		gatherableQuery = GetEntityQuery(ComponentType.ReadOnly<Gatherable>(), ComponentType.ReadOnly<Translation>(), ComponentType.ReadOnly<CompositeScale>());
		bufferSystem = World.GetOrCreateSystem<BeginInitializationEntityCommandBufferSystem>();
	}
	protected override JobHandle OnUpdate(JobHandle inputDeps)
	{
		
		using (var TrailFinished = trailLoopQuery.ToComponentDataArray<TrailFinished>(Allocator.TempJob))
		{
			int trailCount = trailLoopQuery.CalculateEntityCount();
			if (trailCount > 0)
			{
				var TrailLoopPointEntities = trailLoopQuery.ToEntityArray(Allocator.TempJob);
				runningId += 1;
				var cmdBuffer = bufferSystem.CreateCommandBuffer();
				var attractorEnt = cmdBuffer.CreateEntity();
				cmdBuffer.AddComponent(attractorEnt, new AttractionPoint() { id = runningId, requiredMass = (int)LevellingData.Inst.requiredMass.Evaluate(LevellingData.Inst.currentScore), requiredDistance = LevellingData.Inst.requiredMass.Evaluate(LevellingData.Inst.currentScore), creationTime = Time.ElapsedTime});
				cmdBuffer.AddComponent(attractorEnt, new Translation() { Value = TrailFinished[0].midPoint });
				

				GatherableJob job = new GatherableJob()
				{
					trailEntities = TrailLoopPointEntities,
					pointBuffers = GetBufferFromEntity<TrailLoopPoint>(true),
					buffer = bufferSystem.CreateCommandBuffer().ToConcurrent(),
					attractionPointId = runningId,
					midPoint = TrailFinished[0].midPoint,
					attractors = GetComponentDataFromEntity<GatherableAttractor>(true)
				};
				return job.ScheduleSingle(gatherableQuery, inputDeps);
			}
		}
		return inputDeps;
	}	
}
